function NetView(varargin)%set,annot)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Set up background image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%SPM maximum intensity projection in MNI space (mip96)
load(['icons',filesep,'MIP']);
img=mask_all+grid_all; %img=mip96+mask_all;
%Eliminate grid lines outside brain
ind_grid_outside_brain=find(img==2);
img=img+mask_all;
img(ind_grid_outside_brain)=1;
%Set background
tmp=ones(size(img));
for i=26:198
    for j=22:157
        tmp(i,j)=0;
    end
    for j=212:336
        tmp(i,j)=0;
    end
end
for i=240:375
    for j=212:336
        tmp(i,j)=0;
    end
end
%Set background color
%*0 = white
%*3 = black
img=img+tmp*0;

%Colormap
C=[1 1 1;         %white: brain
    0.7 0.7 0.7    %gray: grid lines in brain and non-brain
    0.4 0.4 0.4];  %dark gray: outside brain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Set up figure and figure sizing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Image aspect
Aspect=size(img);

%Screen aspect
sz=get(0,'ScreenSize');
ratio=sz(4)/sz(3);
S.fh3D=-1;
S.fh = figure('units','normalized',...
    'position',[0.055 0.055 1-0.2 1-0.2],...
    'menubar','none',...
    'name','Network View',...
    'numbertitle','off',...
    'color','white',...
    'resize','on',...
    'colormap',C);

offset=0;
cntr_x=0.5;
cntr_y=0.5;

max_x=1-2*offset;
max_y=1-2*offset;
width=Aspect(1);
height=Aspect(2);

if height<width
    orig_x=cntr_x-max_x/2*ratio;
    orig_y=cntr_y-max_y/2*(height/width)*(max_x/max_y);
    len_x=max_x*ratio;
    len_y=max_y*(height/width)*(max_x/max_y);
else
    orig_y=cntr_y-max_y/2;
    orig_x=cntr_x-max_x/2*(width/height)*(max_y/max_x)*ratio;
    len_x=max_x*(width/height)*(max_y/max_x)*ratio;
    len_y=max_y;
end
P.mip=[orig_x orig_y len_x len_y];

%Determined from mip96
P.list=[240 Aspect(2)-343];
P.list(1)=orig_x+P.list(1)/width*len_x;
P.list(2)=orig_y+P.list(2)/height*len_y;
x_end=orig_x+375/width*len_x;
y_end=orig_y+(Aspect(2)-208)/height*len_y;
P.list(3)=x_end-P.list(1);
P.list(4)=y_end-P.list(2);
%P.list(1)=0.6;
P.list(3)=0.3;
P.mip(1)=0.05;
P.mip(3)=0.9;
S.ax = axes('units','normalized',...
    'position',P.mip,...
    'box','off');

S.ls = uicontrol('style','list',...
    'unit','normalized',...
    'position',P.list,...
    'min',0,'max',2,...
    'fontsize',12,...
    'BackgroundColor','white',...
    'Value',[],...
    'String','Load network...');

%Set up toolbar buttons
S.toolbar=uitoolbar(S.fh);
load(['icons',filesep,'glicons']);

[X map] = imread('icons\save_file.gif');
icon = ind2rgb(X,map);
S.button(1)=uipushtool(S.toolbar,...
    'TooltipString','Save Figure',...
    'CData',icon);



S.button(2)=uipushtool(S.toolbar,...
    'TooltipString','Decrease Node Size',...
    'CData',icons.downblue);
S.button(3)=uipushtool(S.toolbar,...
    'TooltipString','Increase Node Size',...
    'CData',icons.upblue);
S.button(4)=uipushtool(S.toolbar,...
    'TooltipString','Decrease Line Width',...
    'CData',icons.downdarkblue);
S.button(5)=uipushtool(S.toolbar,...
    'TooltipString','Increase Line Width',...
    'CData',icons.updarkblue);
S.button(6)=uipushtool(S.toolbar,...
    'TooltipString','Reset Sizing',...
    'CData',icons.flip);
% S.button(6)=uitoggletool(S.toolbar,...
%            'TooltipString','Show All Nodes',...
%            'CData',icons.tree);

[X map] = imread('icons\3D.gif');
icon = ind2rgb(X,map);

S.button(7)=uipushtool(S.toolbar,...
    'TooltipString','Show In 3D',...
    'CData',icon);
[X map] = imread('icons\BrainNet.gif');
icon2 = ind2rgb(X,map);
S.button(8)=uipushtool(S.toolbar,...
    'TooltipString','Save in BrainNet Format',...
    'CData',icon2);

S.IH_sag = imagesc(rot90(img),'Parent',S.ax);  % Display the image.
axis image;
set(S.ax,'xtick',[],'ytick',[],'box','off','Visible','off');
hold on;

%Remove highlighting from listbox
set(S.ls,'Callback',{@bdfcn_ls,S});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Display first network if NET structure exists
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin>0
    %Draw first network
    NET=varargin{1};
    if isfield(NET,'NET')
        NET=NET.NET;
    end
    n=1;
    try draw_net(S,NET,n);
    catch err;
        msg='Does not contain valid network structure';
        tmp=get(S.ls,'string');
        set(S.ls,'string',[{msg};tmp]);
    end
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Close figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function close_fig(varargin)
S=varargin{3};
delete(S.fh);
delete(S.fh3D);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Adjust network display properties
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function decrease_node_size(varargin)
S=varargin{3};
if isfield(S,'mark_axl')
    nw=get(S.mark_axl(1),'MarkerSize');
    if nw>3
        nw=nw-3;
        set(S.mark_axl(:),'MarkerSize',nw);
        set(S.mark_sag(:),'MarkerSize',nw);
        set(S.mark_cor(:),'MarkerSize',nw);
    end
end
function increase_node_size(varargin)
S=varargin{3};
if isfield(S,'mark_axl')
    nw=get(S.mark_axl(1),'MarkerSize');
    nw=nw+3;
    set(S.mark_axl(:),'MarkerSize',nw);
    set(S.mark_sag(:),'MarkerSize',nw);
    set(S.mark_cor(:),'MarkerSize',nw);
end
function decrease_line_width(varargin)
S=varargin{3};
if isfield(S,'line_axl')
    lw=get(S.line_axl(1),'LineWidth');
    if lw>0.5
        lw=lw-0.5;
        set(S.line_axl(:),'LineWidth',lw);
        set(S.line_sag(:),'LineWidth',lw);
        set(S.line_cor(:),'LineWidth',lw);
    end
end
function increase_line_width(varargin)
S=varargin{3};
if isfield(S,'line_axl')
    lw=get(S.line_axl(1),'LineWidth');
    lw=lw+0.5;
    set(S.line_axl(:),'LineWidth',lw);
    set(S.line_sag(:),'LineWidth',lw);
    set(S.line_cor(:),'LineWidth',lw);
end

function show3D(varargin)

%scatter3([1,100],[1,100],[1,100]);

S=varargin{3};
if(S.fh3D>0)
    currView=get(S.fh3D);
    if(~isempty(currView))
        delete(currView)
    end
end
S.fh3D=figure('Name','3D Network View','NumberTitle','off');

Plot3DVol;
hold on;
Net=varargin{4};
node_annot=varargin{5};
node_coor=varargin{6};
x=node_coor(:,1);
y=node_coor(:,2);
z=node_coor(:,3);
node_coor(:,1)=y+120;
node_coor(:,2)=x+91;
node_coor(:,3)=z+70;
allAnnots = unique(node_annot(:,2));
colors=varargin{7};
rmPos=find(allAnnots==0);
if(rmPos>0)
    allAnnots(rmPos)=[];
end
for i=1:Net.N
    pos=find(node_annot(:,1)==i);
    annotOf_i= node_annot(pos(1),2);
    annotPos=find(allAnnots==annotOf_i)
    colorInd=annotOf_i;
    if(colorInd>size(colors,1))
        colorInd=annotPos;
    end
    node_i_col=colors(colorInd,:);
    % if(~ismember(annotPos,usedColors))
    %    usedColors=[usedColors,annotPos];
    %end
    %Only draw node if a line orignates from it
    if ismember(i,[Net.ind_i;Net.ind_j])
        line(node_coor(i,1),node_coor(i,2),node_coor(i,3),...
            'Marker','.','MarkerSize',Net.nw,'Color',node_i_col);
    else
        line(node_coor(i,1),node_coor(i,2),node_coor(i,3),...
            'Visible','off','Marker','.','MarkerSize',Net.nw,'Color',node_i_col);
        
    end
    
end
for i=1:length(Net.ind_i)
    line([node_coor(Net.ind_i(i),1),node_coor(Net.ind_j(i),1)],...
        [node_coor(Net.ind_i(i),2),node_coor(Net.ind_j(i),2)],...
        [node_coor(Net.ind_i(i),3),node_coor(Net.ind_j(i),3)],...
        'LineWidth',Net.lw,...
        'Color',Net.line_color);
end



function reset(varargin)
S=varargin{3};
Net=varargin{4};
if isfield(S,'line_axl')
    set(S.line_axl(:),'LineWidth',Net.lw);
    set(S.line_sag(:),'LineWidth',Net.lw);
    set(S.line_cor(:),'LineWidth',Net.lw);
end
if isfield(S,'mark_axl')
    set(S.mark_axl(:),'MarkerSize',Net.nw);
    set(S.mark_sag(:),'MarkerSize',Net.nw);
    set(S.mark_cor(:),'MarkerSize',Net.nw);
end
function show_all_nodes(varargin)
S=varargin{3};
if isfield(S,'mark_axl')
    for i=1:length(S.mark_axl)
        if strcmp('off',get(S.mark_axl(i),'Visible'))
            set(S.mark_axl(i),'Visible','on','UserData',1);
            set(S.mark_sag(i),'Visible','on','UserData',1);
            set(S.mark_cor(i),'Visible','on','UserData',1);
        end
    end
end
function undo_show_all_nodes(varargin)
S=varargin{3};
if isfield(S,'mark_axl')
    for i=1:length(S.mark_axl)
        if get(S.mark_axl(i),'UserData')
            set(S.mark_axl(i),'Visible','off','UserData',0);
            set(S.mark_sag(i),'Visible','off','UserData',0);
            set(S.mark_cor(i),'Visible','off','UserData',0);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Draw network
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function draw_net(varargin)
if length(varargin)>4
    S=varargin{3};
    NET=varargin{4};
    n=varargin{5};
else
    S=varargin{1};
    NET=varargin{2};
    n=varargin{3};
end
%Delete any previous dropdown menu
delete(get(S.ax,'UserData'));
set(S.ax,'UserData',[]);

if ~isempty(NET.n) && NET.n>0;
    
    %Only create a dropdown menu if more than one signficant network
    %and dropdown menu not previously created
    if NET.n>1 %&& isempty(get(S.ax,'UserData'))
        S.menu_view=uimenu(S.fh,'Label','View');
        for i=1:NET.n
            S.menu_view_net(i)=uimenu(S.menu_view,'Label',['Network ',num2str(i)],...
                'Callback',{@draw_net,S,NET,i});
        end
        %Use UserData of the axes object to indicate whether a menu has been
        %previously set up
        set(S.ax,'UserData',S.menu_view);
    end
    
    %Origins in MIP
    origin.axl=[126 275];
    origin.sag=[126 111];
    origin.cor=[308 111];
    for i=1:size(NET.node_coor,1)
        Coor.x_axl(i)=origin.axl(1)+NET.node_coor(i,2);
        Coor.y_axl(i)=origin.axl(2)+NET.node_coor(i,1);
        Coor.x_sag(i)=origin.sag(1)+NET.node_coor(i,2);
        Coor.y_sag(i)=origin.sag(2)-NET.node_coor(i,3);
        Coor.x_cor(i)=origin.cor(1)+NET.node_coor(i,1);
        Coor.y_cor(i)=origin.cor(2)-NET.node_coor(i,3);
    end
    
    %Determine indices of edges
    if isfield(NET,'con_mat')
        if iscell(NET.con_mat)
            [ind_i,ind_j]=find(NET.con_mat{n});
        else
            [ind_i,ind_j]=find(NET.con_mat);
        end
    else
        ind_i=[];
        ind_j=[];
    end
    %Line width
    lw=1.5;
    %Default line color
    line_color=[0 0 0];
    %Node width
    nw=24;%30;
    %Default node color
    node_color=[0 0 0];
    %Make a structure
    Net.lw=lw; Net.line_color=line_color; Net.nw=nw; Net.node_color=node_color;
    
    if isfield(NET,'con_mat')
        
        if iscell(NET.con_mat)
            Net.clicked_node=spalloc(length(NET.con_mat{n}),1,1);
            Net.clicked_line=spalloc(length(NET.con_mat{n}),length(NET.con_mat{n}),1);
            % Net.clicked_line=spalloc(allocation,allocation,1);
            Net.N=length(NET.con_mat{n});
        else
            Net.clicked_node=spalloc(length(NET.con_mat),1,1);
            Net.clicked_line=spalloc(length(NET.con_mat),length(NET.con_mat),1);
            %Net.clicked_line=spalloc(allocation,allocation,1);
            Net.N=length(NET.con_mat);
        end
    end
    if isfield(NET,'test_stat')
        Net.test_stat=NET.test_stat;
    end
    
    Net.ind_i=ind_i;
    Net.ind_j=ind_j;
    
    %Delete previous network
    prev_handles=get(S.fh,'UserData');
    if ~isempty(prev_handles)
        %Delete handles
        delete(prev_handles(:));
    end
    if isfield( S,'fh3D')
        if(S.fh3D>0)
            prev_handles=get(S.fh3D,'UserData');
            if ~isempty(prev_handles)
                %Delete handles
                delete(prev_handles(:));
            end
        end
    end
    
    %Reset toggle to off
    %          set(S.button(6),'OnCallback','');
    %          set(S.button(6),'OffCallback','');
    %          set(S.button(6),'State','off');
    edges=length(Net.ind_i);
    selfEdges=0;
    if isfield(NET,'con_mat')
        selfEdges=sum(diag(NET.con_mat>0));
    end
    edges=edges-selfEdges;
    %Put information in list box
    if isfield(NET,'pval')
        pval=NET.pval(n);
        if pval<0.001
            s_pval='<0.001';
        else
            s_pval=sprintf('%0.3f',pval);
        end
        
        s={NET.title;...
            ['p-value: ',s_pval];...
            ['Frequency ratio: ',num2str(NET.freqRat)];...
            ['Number of edges: ',num2str(edges)];...
            ['Number of nodes: ',num2str(length(unique([Net.ind_i;Net.ind_j])))];...
            'Click on a node or edge...'};
    else
        s={NET.title;...
            ['Number of edges: ',num2str(edges)];...
            ['Number of nodes: ',num2str(length(unique([Net.ind_i;Net.ind_j])))];...
            'Click on a node or edge...'};
    end
    set(S.ls,'String',s);
    
    %colors=hsv(60);
    %Draw edges
    [S.line_axl,S.line_sag,S.line_cor]=draw_edges(Coor,Net);
    %Draw nodes
    [S.mark_axl,S.mark_sag,S.mark_cor]=draw_nodes(Coor,Net,NET.node_annot,NET.colors);
    %Assign callback to nodes
    for i=1:length(S.mark_axl)
        set(S.mark_axl(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
        set(S.mark_sag(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
        set(S.mark_cor(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
    end
    %Assign callback to each line
    for i=1:length(S.line_axl)
        set(S.line_axl(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
        set(S.line_sag(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
        set(S.line_cor(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
    end
    %Assign callback to toolbar buttons
    set(S.button(1),'ClickedCallback',{@saveView,S});
    set(S.button(2),'ClickedCallback',{@decrease_node_size,S});
    set(S.button(3),'ClickedCallback',{@increase_node_size,S});
    set(S.button(4),'ClickedCallback',{@decrease_line_width,S});
    set(S.button(5),'ClickedCallback',{@increase_line_width,S});
    set(S.button(6),'ClickedCallback',{@reset,S,Net});
    % set(S.button(6),'OnCallback',{@show_all_nodes,S});
    % set(S.button(6),'OffCallback',{@undo_show_all_nodes,S});
    set(S.button(7),'ClickedCallback',{@show3D,S,Net,NET.node_annot,NET.node_coor,NET.colors});
    set(S.button(8),'ClickedCallback',{@export2BrainNet,S,Net,NET.node_annot,NET.node_coor,NET.colors,NET.annotConv,NET.title});
    %Store line and node handles in UserData of the figure
    %This allows subsequnt deletion of the lines and nodes
    set(S.fh,'UserData',[S.mark_axl,S.mark_sag,S.mark_cor,...
        S.line_axl,S.line_sag,S.line_cor]);
    
else
    set(S.ls,'String','No network to display');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Draw edges
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [line_axl,line_sag,line_cor]=draw_edges(Coor,Net)
line_axl=[];
line_sag=[];
line_cor=[];
if(length(Net.ind_i)>0)
    
    for i=1:length(Net.ind_i)
        line_axl(i)=line([Coor.x_axl(Net.ind_i(i)),Coor.x_axl(Net.ind_j(i))],...
            [Coor.y_axl(Net.ind_i(i)),Coor.y_axl(Net.ind_j(i))],...
            'LineWidth',Net.lw,...
            'Color',Net.line_color);
        line_sag(i)=line([Coor.x_sag(Net.ind_i(i)),Coor.x_sag(Net.ind_j(i))],...
            [Coor.y_sag(Net.ind_i(i)),Coor.y_sag(Net.ind_j(i))],...
            'LineWidth',Net.lw,...
            'Color',Net.line_color);
        line_cor(i)=line([Coor.x_cor(Net.ind_i(i)),Coor.x_cor(Net.ind_j(i))],...
            [Coor.y_cor(Net.ind_i(i)),Coor.y_cor(Net.ind_j(i))],...
            'LineWidth',Net.lw,...
            'Color',Net.line_color);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Draw nodes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [mark_axl,mark_sag,mark_cor]=draw_nodes(Coor,Net,node_annot,colors)
allAnnots = unique(node_annot(:,2));
usedColors=[];
%    drawn=[];
%     if(Net.N<=500)
drawn=1:Net.N;
%     else
%         dThresh=3;
%         for i=1:Net.N
%             skip=0;
%
%             for d=1:length(drawn)
%                 other=drawn(d);
%
%                axlDist=pdist2([Coor.x_axl(i),Coor.y_axl(i)],[Coor.x_axl(other),Coor.y_axl(other)]);
%                sagDist=pdist2([Coor.x_sag(i),Coor.y_sag(i)],[Coor.x_sag(other),Coor.y_sag(other)]);
%                corDist=pdist2([Coor.x_cor(i),Coor.y_cor(i)],[Coor.x_cor(other),Coor.y_cor(other)]);
%                if(axlDist<dThresh && sagDist<dThresh && corDist<dThresh)
%                    skip=1;
%
%                    break;
%                end
%
%             end
%             if(~skip)
%                 drawn=[drawn,i];
%             end
%         end
%     end


for d=1:length(drawn)%Net.N
    i=drawn(d);
    
    pos=find(node_annot(:,1)==i);
    annotOf_i= node_annot(pos(1),2);
    annotPos=find(allAnnots==annotOf_i)
    colorInd=annotOf_i;
    if(colorInd>size(colors,1))
        colorInd=annotPos;
    end
    node_i_col=colors(colorInd,:);
    if(~ismember(annotPos,usedColors))
        usedColors=[usedColors,annotPos];
    end
    
    %Only draw node if a line orignates from it
    %    if ismember(i,[Net.ind_i;Net.ind_j])
    %        mark_axl(i)=line(Coor.x_axl(i),Coor.y_axl(i),...
    %            'Marker','.','MarkerSize',Net.nw,'Color',node_i_col);
    %        mark_sag(i)=line(Coor.x_sag(i),Coor.y_sag(i),...
    %            'Marker','.','MarkerSize',Net.nw,'Color',node_i_col);
    %        mark_cor(i)=line(Coor.x_cor(i),Coor.y_cor(i),...
    %            'Marker','.','MarkerSize',Net.nw,'Color',node_i_col);
    
    %    else
    mark_axl(i)=line(Coor.x_axl(i),Coor.y_axl(i),...
        'Marker','.','MarkerSize',Net.nw,...
        'Visible','off','Color',node_i_col);
    mark_sag(i)=line(Coor.x_sag(i),Coor.y_sag(i),...
        'Marker','.','MarkerSize',Net.nw,...
        'Visible','off','Color',node_i_col);
    mark_cor(i)=line(Coor.x_cor(i),Coor.y_cor(i),...
        'Marker','.','MarkerSize',Net.nw,...
        'Visible','off','Color',node_i_col);
    
    %   end
    
end
set(mark_cor,'Visible','on');
set(mark_sag,'Visible','on');
set(mark_axl,'Visible','on');
for i=1:length(usedColors)
    %rectangle('Position',[0.7,0.3,0.05,0.05],'EdgeColor','k','FaceColor',colors(usedColors(i),:),'Visible','on');
    line(0.7,0.3,...
        'Marker','.','MarkerSize',Net.nw,...
        'Visible','off','Color',colors(usedColors(i),:));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Disable highlight in listbox
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bdfcn_ls(varargin)
S = varargin{3};
set(S.ls,'Value',[]);

function clearSel(varargin)
S=varargin{1};
Net=varargin{2};

%[S,i,Net,NET] = varargin{[3,4,5,6]};
sel_node=find(Net.clicked_node);
[sel_edge1,sel_edge2]=find(Net.clicked_line);
if(isempty(sel_node) && isempty(sel_edge1))
    return;
end
ls=get(S.ls,'String');
set(S.ls,'String',ls(1:end-1,:));

set(S.mark_axl(sel_node),'Color',Net.node_color);
set(S.mark_sag(sel_node),'Color',Net.node_color);
set(S.mark_cor(sel_node),'Color',Net.node_color);
set(S.line_axl(:),'Color',Net.line_color);
set(S.line_sag(:),'Color',Net.line_color);
set(S.line_cor(:),'Color',Net.line_color);
Net.clicked_node(sel_node)=0;
Net.clicked_line(sel_edge1,sel_edge2)=0;
[sel_edge1,sel_edge2]=find(Net.clicked_line);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Buttondown for node
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=bdfcn_node(varargin)
clearSel(varargin{[3,5]});
% src - the object that is the source of the event
% evnt - empty for this property
[S,node,Net,NET] = varargin{[3,4,5,6]};
sel_typ = get(gcbf,'SelectionType');
%disp('User clicked left-mouse button')
%Nothing previously clicked
Net.node_color=get(S.mark_axl(node),'Color');
set(S.mark_axl(node),'Color','red');
set(S.mark_sag(node),'Color','red');
set(S.mark_cor(node),'Color','red');
if isfield(NET,'node_label')
    ls=get(S.ls,'String');
    set(S.ls,'String',[ls;{NET.node_label{node}}]);
else
    ls=get(S.ls,'String');
    set(S.ls,'String',[ls;{' '}]);
end
Net.clicked_node(node)=1;


for i=1:length(S.mark_axl)
    set(S.mark_axl(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
    set(S.mark_sag(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
    set(S.mark_cor(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
end
for i=1:length(S.line_axl)
    set(S.line_axl(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
    set(S.line_sag(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
    set(S.line_cor(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Buttondown for edge
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function []=bdfcn_line(varargin)
clearSel(varargin{[3,5]});
% src - the object that is the source of the event
% evnt - empty for this property
[S,i,Net,NET] = varargin{[3,4,5,6]};
sel_typ = get(gcbf,'SelectionType');

switch sel_typ
    case 'normal'
        %disp('User clicked left-mouse button')
        %   str=NET.title;
        if isfield(Net,'test_stat') && isfield(NET,'node_label')
            tstat=sprintf(' %0.2f',Net.test_stat(Net.ind_i(i),Net.ind_j(i)));
            str=[ '[' NET.node_label{Net.ind_i(i)},'] [',NET.node_label{Net.ind_j(i)},']; test stat:',tstat];
        elseif isfield(NET,'node_label')
            str=[ '[' NET.node_label{Net.ind_i(i)},'] [',NET.node_label{Net.ind_j(i)} ']'];
        elseif isfield(Net,'test_stat')
            str=sprintf( 'test stat: %0.2f',Net.test_stat(Net.ind_i(i),Net.ind_j(i)));
        else
            str=' ';
        end
        set(S.line_axl(i),'Color','red');
        set(S.line_sag(i),'Color','red');
        set(S.line_cor(i),'Color','red');
        ls=get(S.ls,'String');
         set(S.ls,'String',[ls;{str}]);
         Net.clicked_line(Net.ind_i(i),Net.ind_j(i))=1;
                %Update Net
        for i=1:length(S.mark_axl)
            set(S.mark_axl(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
            set(S.mark_sag(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
            set(S.mark_cor(i),'buttondownfcn',{@bdfcn_node,S,i,Net,NET});
        end
        for i=1:length(S.line_axl)
            set(S.line_axl(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
            set(S.line_sag(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
            set(S.line_cor(i),'buttondownfcn',{@bdfcn_line,S,i,Net,NET});
        end
    case 'extend'
        aaa=1;
        %disp('User did a shift-click');
        %set(src,'Selected','on')
    case 'alt'
        aaa=1;
        %disp('User did a control-click');
        %set(src,'Selected','on')
        %set(src,'SelectionHighlight','off')
end
        
        
        function  saveView(varargin)
        
        S=varargin{3};
        [filename, ext, user_canceled] = imputfile ;
        if(user_canceled>0)
            return;
        end
        try
            saveas(S.fh,filename,ext);
        catch exception
            msgbox(exception.message);
        end
        
%Exporting 3D network to BrainNet format
        function export2BrainNet(varargin)
        try
        dirPath=uigetdir('','Save BrainNet input files in ');  
        if(dirPath==0)
            return;
        end
        S=varargin{3};
  
        Net=varargin{4};
        node_annot=varargin{5};
        node_coor=varargin{6};
       
        allAnnots = unique(node_annot(:,2));
        colors=varargin{7};
        %netName='enrichmentInducing';
        rmPos=find(allAnnots==0);
        if(rmPos>0)
            allAnnots(rmPos)=[];
        end
        col4=zeros(Net.N,1);
        col5=ones(Net.N,1);
        col6=cell(Net.N,1);
        conMat=zeros(Net.N);
        for i=1:Net.N
            pos=find(node_annot(:,1)==i);
            annotOf_i= node_annot(pos(1),2);
            annotPos=find(allAnnots==annotOf_i)
            colorInd=annotOf_i;
            col4(i)=colorInd;
            if(colorInd>size(colors,1))
                colorInd=annotPos;
            end
            annotConv=varargin{8};
            netName=varargin{9};
            netName=strrep(netName, ':','_');
            netName=strrep(netName, '-','_');
            netName=strrep(netName, ' ','_');
            
            if(~isempty(annotConv))
                annotNum=cell2mat(annotConv(:,1));
                annotPos=find(annotNum==annotOf_i);
                col6{i}=annotConv{annotPos,2};
            else
               col6{i}=annotOf_i;
            end
           
        end
        nodeData=num2cell([node_coor,col4,col5]);
        nodeData=[nodeData,col6];
        %dlmwrite([dirPath '\' netName '.node'],nodeData,'\t');
        dlmcell([dirPath '\' netName '.node'],nodeData); 
        for con=1:length(Net.ind_i)
            conMat(Net.ind_i(con),Net.ind_j(con))=1;
            conMat(Net.ind_j(con),Net.ind_i(con))=1;
        end
    dlmwrite([dirPath '\' netName '.edge'],conMat,'\t');
    msgbox(['BrainNet files successfully exported into ' dirPath]);
    catch exception
        msgbox(exception.message);

    end