function p = hygecdf_(x,m,k,n, logBase2Use)
%HYGECDF Hypergeometric cumulative distribution function.
%   P = HYGECDF(X,M,K,N) returns the hypergeometric cumulative
%   distribution function with parameters M, K, and N
%   at the values in X.
%
%   The size of P is the common size of the input arguments. A scalar input
%   functions as a constant matrix of the same size as the other inputs.
%
%   See also HYGEINV, HYGEPDF, HYGERND, HYGESTAT, CDF.

%   Copyright 1993-2010 The MathWorks, Inc.
%   $Revision: 1.1.8.5 $  $Date: 2010/10/08 17:24:25 $

if nargin < 4,
    error(message('stats:hygecdf:TooFewInputs'));
end

p=0;
for i=0:(x)
    if(i>k || n-i>m-k || n>m)
        addition=0;
    else
        if(logBase2Use<1)
            addition_ln=nchoosekln(k,i)+nchoosekln(m-k,n-i)-nchoosekln(m,n);
            addition=exp(addition_ln);
        else
            base=logBase2Use;
            addition_log=lognchoosek(k,i,base)+lognchoosek(m-k,n-i,base)-lognchoosek(m,n,base);
            addition=power(base,addition_log);
        end
    end
    p=p+addition;
end
if(p>1)
    p=1;
end